#!/usr/bin/env python

import matplotlib.animation as animation
import numpy as np
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider

ω = 100.0
σω = 0.
Γ = 0.

δt = 0.1

t = 0.

xx = np.linspace(-2.0, 2.0, 20001)
Δω = np.linspace(-50.0, 50.0, 2001)


def gauss(σω, ω=ω, Γ=Γ, t=0.):
    if σω == 0.:
        return np.ones_like(xx)
    σ = 1. / σω
    σ2t = (σ**2 + 1j * Γ * t)
    return σ / σ2t**0.5 * np.exp(-0.5 * xx**2 / σ2t)


def fourier_gauss(σω, Γ=Γ, t=0.):
    σω = max(σω, 0.0001)
    return np.abs(np.exp(-0.5 * Δω**2 * (σω**(-2) + 1j * Γ * t)))


fig, (ax1, ax2) = plt.subplots(nrows=2)

manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Oscylator z wymuszeniem prostokątnym")
manager.full_screen_toggle()

line1, = ax1.plot(ω + Δω, fourier_gauss(σω), 'C1', lw=1.5)
ax1.set_xlim(ω + Δω[0], ω + Δω[-1])
ax1.set_xlabel('$\\omega$')
ax1.set_ylim(-0.02, 1.02)

psi = gauss(σω)
phas = np.exp(1j * ω * xx)
line2, = ax2.plot(xx, psi * phas.real, lw=1.5)
line2e1, = ax2.plot(xx, psi, 'k--', lw=0.5)
line2e2, = ax2.plot(xx, -psi, 'k--', lw=0.5)
ax2.set_xlim(xx[0], xx[-1])
ax2.set_xlabel('$x - c_g t$')
ax2.set_ylim(-1.02, 1.02)

σax = fig.add_axes([0.17, 0.07, 0.75, 0.03])
ωax = fig.add_axes([0.17, 0.04, 0.75, 0.03])
cax = fig.add_axes([0.17, 0.01, 0.45, 0.03])
Γax = fig.add_axes([0.67, 0.01, 0.25, 0.03])

fig.subplots_adjust(bottom=0.19, top=0.98, left=0.08, right=0.98)

sst = {
    'track_color': '0.7',
    'color': '0.7',
    'initcolor': 'black',
}

omega_slider = Slider(ax=ωax, label='$\\omega$', valmin=0., valmax=200., valinit=ω, valstep=1., **sst)
sigma_slider = Slider(ax=σax, label='$\\sigma_\\omega$', valmin=0., valmax=20.0, valinit=σω, valstep=0.001, **sst)
deltac_slider = Slider(ax=cax, label='$c_f - c_g$', valmin=-1., valmax=1., valinit=0., valstep=0.1, **sst)
Gamma_slider = Slider(ax=Γax, label='$\\Gamma$', valmin=-0.01, valmax=0.01, valinit=0., valstep=0.0002, **sst)


def on_slider(val=None):
    global psi, phas, ω, t
    t = 0.
    phas = np.exp(1j * ω * xx)
    ω = omega_slider.val
    line1.set_data(ω + Δω, fourier_gauss(sigma_slider.val))
    psi = gauss(sigma_slider.val, ω)
    line2.set_ydata(psi * phas.real)
    line2e1.set_ydata(psi)
    line2e2.set_ydata(-psi)
    ax1.set_xlim(ω + Δω[0], ω + Δω[-1])
    yl = 1.02 * np.max(np.abs(psi.real))
    # ax2.set_ylim(-yl, yl)
    fig.canvas.draw_idle()


def animate(i):
    global phas, t, psi
    t += δt
    phas *= np.exp(-1j * deltac_slider.val * δt)
    Γ = Gamma_slider.val
    if Γ != 0.:
        psi = gauss(sigma_slider.val, ω, Γ, t)
        line1.set_ydata(fourier_gauss(sigma_slider.val, Γ, t))
        aps = np.abs(psi)
        line2e1.set_ydata(aps)
        line2e2.set_ydata(-aps)
    line2.set_ydata((psi * phas).real)
    return line1, line2, line2e1, line2e2


anim = animation.FuncAnimation(fig, animate, interval=10, blit=True, cache_frame_data=False)

omega_slider.on_changed(on_slider)
sigma_slider.on_changed(on_slider)
deltac_slider.on_changed(on_slider)
Gamma_slider.on_changed(on_slider)

paused = False


def toggle_pause(ev):
    if ev.key == 'r':
        on_slider()
    if ev.key != ' ': return
    global paused
    if paused:
        anim.resume()
    else:
        anim.pause()
    paused = not paused


fig.canvas.mpl_connect('key_press_event', toggle_pause)


plt.show()
